// Objekt zur Steuerung der amira DAC98 Karte
// Stand 12.11.1997 B.Osterkamp
// Dies ist die Header-Datei

/* Adressdefinitionen */

/* Adressen der Chips */
#define DAC98ID		0x0055	// HardwareIdentitt der DAC98

#define Iden_CS	  	0x0000	// /CS0  : Identifikation
#define ADW_CS		0x0008	// /CS1  : ADW
#define DAW0_CS		0x0010	// /CS2  : DAW0
#define DAW1_CS		0x0018	// /CS3  : DAW1
#define Timer_CS	0x0020	// /CS4  : Timer
#define IO_CS  		0x0028	// /CS5  : IO
#define DDM02a_CS  	0x0030	// /CS6  : DDM02a
#define DDM02b_CS  	0x0038	// /CS7  : DDM02b
#define DDM02c_CS  	0x0078	// /CS8  : DDM02c
#define CSDDMALL_CS  	0x00BA	// /CS9  : CSDDMALL
#define Interrupt_CS	0x00F8	// /CS10 : Interrupt

/* DDM-Register */
#define DDMINCRHI	0x00
#define DDMINCRLO	0x01

unsigned int    ddm_counter[3];

enum Clkmodes {Clk8MHz=0,Clk4MHz=1,Clk2MHz=2,Clk1MHz=3,Clk500kHz=4,Clk250kHz=5,Clk125kHz=6,Clk62kHz=7};

int aout0, aout1;



unsigned char	ddm_adr[3];
unsigned long   timer_counter0,timer_counter1;
unsigned int	timer_counter2;

int Base	;	// Basisadresse der DAC98

int WR_DATA 	;	// Datenregister schreiben
int RD_DATA 	;	// Datenregister auslesen
int intr	;
double Clock    ;
int CounterGate ;
int CounterJMP	;

// Variablen der Digitalen Kanle
int output_status_DAC98;// Registerzustand der digitalen Ausgnge
int input_status_DAC98; // Registerzustand der digtialen Eingnge
int intr_status;        // Zustand des Interruptsregisters



	// Einlesen der digitalen Eingnge
	unsigned int ReadDigitalInputs( void );


	// Der Konstruktor bentigt nur die Basisadresse
	DAC98( int adress = 0x300 );


	// liefert die eingestellte Basisadresse zurck
	int GetAdress( void ) { return Base; };

	// setzt die Basisadresse auf einen neuen Wert
	void SetAdress( int adress ) { Base = adress; };

	// liefert den eingestellten Interruptkanal zurck
	int GetInterrupt( void ) { return intr; };

	// setzt den Interrupt auf einen neuen Wert
	void SetInterrupt( int i ) { intr = i; };

	// Auslesen der Identifikation-Bitfolge
	int Identifikation( void );

	// Initialisieren der Bausteine
	void Init ( void );

	// Karte in den Grundzustand bringen
	int Exit ( void );

	// Adresse der DAC98 suchen
	int Setup (void);

	// setzt den internen Clockmode fr den Timer/Counter
	void SetClock( int mode );

	// liefert den eingestellten Clockmode zurck
	double GetClock( void ) {return Clock;};

	// Kanalweise die Digitalen Kanle schreiben
	void WriteDigital (int channel, int value);

	// Alle Digitalen Kanle auf val setzen
	void WriteAllDigital( int value );

	// einen Digital-Eingang lesen
	int ReadDigital( int channel );

	// Einlesen der digitalen Eingnge
	int ReadAllDigital( void ){return (int)ReadDigitalInputs();};

	// Den Counter setzen
	void SetCounter(unsigned int count);

	// liefert den Zaehlerstand des Counters.
	unsigned int GetCounter( void );

	// Delayroutine mit Hilfe des Counter, die Zeit wird in ms bergeben
	int WaitCounter(double time);

	// Routine zum berprfen des Frequenzjumpers
	int TestCounterJMP( void );

	// Den Counter starten
	void GateCounter( int val );

	// Den Timer setzen
	void SetTimer ( unsigned long time );

	// Den Timer in ms setzen
	/*void SetTimer( double time )gibt es nur in C++, oder den Namen aendern! */;

	// liefert den Zaehlerstand des Timers.
	unsigned long GetTimer( void );

	// Den Timer starten
	void GateTimer( int val );

	// setzt das Interrupt-Freigaberegister der DAC98
	void SetINT( int channel, int val );

	// liet das Interruptregister zurck
	int GetINT( void );

	// einen DDM zurcksetzen
	void ResetDDM( int channel );

	// alle DDM zurcksetzen
	void ResetAllDDM( void );

	// einen DDM lesen
	unsigned int ReadDDM( int channel );

	// alle DDM lesen
	void ReadAllDDM( void );

	// einen Anaglogen Kanal mit dem entsprechenden Mode lesen
	// default +-10V
	int  ReadAnalogInt( int channel , int mode);

	// einen Anaglogen Kanal mit dem entsprechenden Mode lesen
	// default +-10V mit  der Ausgabe in V
	float  ReadAnalogVolt( int channel , int mode);

	// gibt einen Analogwert auf einem bestimmten Kanal aus
	void WriteAnalogInt( int channel, int value );

	// gibt einen Analogwert in Volt auf einem bestimmten Kanal aus
	void WriteAnalogVolt( int channel, float value );

